package fer.hr.segmentation;

import javax.imageio.ImageIO;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by jcular on 5/8/16.
 */
public class SegmentationThreshold implements ISegmentation {
    @Override
    public List<BufferedImage> extractCharacters(BufferedImage image) {
        int[] rowPixelSum = rowPixelSum(image);
        int tr = findThreshold(rowPixelSum);
        int y1=0, y2=0;

        while(y1<image.getHeight() && y2<image.getHeight()) {
            if (rowPixelSum[y1] <= tr && y1 >= y2) {
                y2 = y1 + 1;
            } else if (y2 > y1 && rowPixelSum[y2] <= tr) {
                y2++;
            } else if (y2 > y1 && rowPixelSum[y2] > tr) {
                break;
            } else{
                y1++;
            }
        }

        BufferedImage horizontalCut = image.getSubimage(0,y1,image.getWidth(),Math.abs(y2-y1));
        int[] columnPixelSum = columnPixelSum(horizontalCut);
        int tc = findThreshold(columnPixelSum);

        List<BufferedImage> characterImages = new ArrayList<>();
        int x1=0,x2=0;

        while(x1<horizontalCut.getWidth() && x2<horizontalCut.getWidth()) {
            if (columnPixelSum[x1] <= tc && x1 >= x2) {
                x2 = x1 + 1;
            } else if (x2 > x1 && columnPixelSum[x2] <= tc) {
                x2++;
            } else if (x2 > x1 && columnPixelSum[x2] > tc) {
                characterImages.add(horizontalCut.getSubimage(x1, 0, x2 - x1, horizontalCut.getHeight()));
                x1 = x2 + 1;
            } else {
                x1++;
            }
        }
        return characterImages;
    }

    private int findThreshold(int[] array) {
        int threshold = 0;
        for (int v : array) {
            threshold += v;
        }
        return (int) Math.round(threshold*1.0/array.length);
    }

    private int[] rowPixelSum(BufferedImage img) {
        int[] rowPixelSum = new int[img.getHeight()];

        for (int i = 0; i < img.getWidth(); i++) {
            for (int j = 0; j < img.getHeight(); j++) {
                rowPixelSum[j] += img.getRGB(i, j) & 0xFF;
            }
        }
        return  rowPixelSum;
    }

    private int[] columnPixelSum(BufferedImage img) {
        int[] columnPixelSum = new int[img.getWidth()];

        for (int i = 0; i < img.getWidth(); i++) {
            for (int j = 0; j < img.getHeight(); j++) {
                columnPixelSum[i] += img.getRGB(i, j) & 0xFF;
            }
        }
        return columnPixelSum;
    }

}
